/*	$Id: basicc.c 2008/01/21 12:03:20 mallet $ */

/*
 * Copyright (c) 2007 Anthony Mallet                  --  Thu Feb  1 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef WIN32
#include <err.h>
#else
#include <stdio.h>

#ifdef __cplusplus
void errx(int eval, const char *fmt) 
{
	printf("%s: \n", fmt);
}
#endif /* __cplusplus */

void errx(int eval, const char *fmt, double ) 
{
	printf("%s: \n", fmt);
}
void warnx(const char *fmt)
{
	printf("%s: \n", fmt);
}
#include <math.h>

#endif /* WIN32 */

#include <t3d/t3d.h>
#include <t3d/v3d.h>

int
main()
{
  T3D s, *t;
  V3D a, b;
  int i;

  if (sizeof(T3D) != sizeof(T3DAny))
    errx(2, "T3D and T3DAny have different sizes");
  warnx("T3D and T3DAny sizes ok");

  /* --- accessing members ------------------------------------------- */

  t3dInit(&s, T3D_BRYAN, 0);
  s.bryan.yaw = 0.0;
  s.bryan.pitch = 1.0;
  s.bryan.roll = 2.0;
  s.bryan.x = 3.0;
  s.bryan.y = 4.0;
  s.bryan.z = 5.0;
  for (i=0; i<6; i++)
    if (s.bryan.bryan[i] != i) errx(2, "bryan member %d inconsistent", i);
  warnx("T3DBryan members access ok");

  t = t3dNew(T3D_MATRIX, 0);
  if (!t) errx(2, "cannot allocate T3D");
  warnx("T3D allocation ok");

  t3dFPrint(stdout, t);
  warnx("T3D print ok");

  v3dFPrint(stdout, &a);
  warnx("V3D print ok");

  s = *t;
  for(i=0;i<16;i++) if (t->matrix.matrix[i] != s.matrix.matrix[i])
    errx(2, "affection not working");
  warnx("affectation ok");

  t3dInvert(&s);
  for(i=0;i<16;i++) if (t->matrix.matrix[i] != s.matrix.matrix[i])
    errx(2, "inversion not working");
  warnx("inversion ok");

  v3dInit(&a);
  b = *v3dApplyT3D(&s, &a);
  for(i=0;i<4;i++) if (b.v[i] != a.v[i])
    errx(2, "vector transformation not working");
  warnx("vector transformation ok");

  /* --- normalize --------------------------------------------------- */

  a.v[0] = a.v[1] = a.v[2] = 2.;
  v3dNormalize(&a);
  if (fabs(v3dNorm(&a) - 1.) > 1e-10)
    errx(2, "vector normalization not working: %f", v3dNorm(&a));
  warnx("vector normalization ok");

  /* --- dot product ---------------------------------------------- */

  a = V3D(3. , -4., 5.);
  b = V3D(-3., 4., 5.);
  double d = v3dDot(&a,&b);
  if (fabs(d) > 1e-10)
    errx(2, "dot product not working");
  warnx("dot product ok");

  /* --- vector product ---------------------------------------------- */

  a.v[0] = 1.; a.v[1] = a.v[2] = a.v[3] = 0.;
  b.v[1] = 1.; b.v[0] = b.v[2] = b.v[3] = 0.;
  v3dProd(&a, &b);
  if (fabs(v3dNorm(&a) - 1.) > 1e-10 ||
      fabs(a.v[2] - 1.) > 1e-10)
    errx(2, "vector product not working: %f", v3dNorm(&a));
  warnx("vector product ok");

  return 0;
}
