/*	$Id: basiccpp.cpp 2007/09/28 15:00:57 mallet $ */

/*
 * Copyright (c) 2007 Anthony Mallet                  --  Thu Feb  1 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <iostream>

#ifndef WIN32
#include <err.h>
#else
void errx(int eval, const char *fmt)
{
	printf("%s: \n", fmt);
}
void errx(int eval, const char *fmt, double ) 
{
	printf("%s: \n", fmt);
}
void warnx(const char *fmt)
{
	printf("%s: \n", fmt);
}
 
#endif /* WIN32 */

#include <math.h>

#include <t3d/t3d.h>
#include <t3d/v3d.h>

int
main()
{
  T3D r, s, *t;
  V3D a, b;
  int i;

  if (sizeof(T3D) != sizeof(T3DAny))
    errx(2, "T3D and T3DAny have different sizes");
  warnx("T3D and T3DAny sizes ok");

  t = new T3D(t3dMatrixIdentity);
  if (!t) errx(2, "cannot allocate CT3D");
  warnx("T3D allocation ok");

  std::cout << "Chained print" << std::endl
	    << *t << "End of chained print" << std::endl;
  warnx("T3D print ok");

  std::cout << a;
  warnx("V3D print ok");

  if (t->matrix[0] != 1.0)
    errx(2, "reading matrix element not working");
  warnx("reading matrix element ok");

  s = *t;
  for(i=0;i<16;i++) if (t->matrix[i] != s.matrix[i])
    errx(2, "affection not working");
  warnx("affectation ok");

  /* --- product ----------------------------------------------------- */

  s = T3D(0., 0., 0., 1., 2., 3.);
  *t = T3D(0., 0., 0., 3., 2., 1.);
  r = s * *t;

  s.convertTo(T3D_BRYAN);
  t->convertTo(T3D_BRYAN);
  r.convertTo(T3D_BRYAN);

  for(i=0;i<6;i++) {
    if (s.bryan.bryan[i] != T3D(0., 0., 0., 1., 2., 3.).bryan.bryan[i])
      errx(2, "multiplication not working");
    if (t->bryan.bryan[i] != T3D(0., 0., 0., 3., 2., 1.).bryan.bryan[i])
      errx(2, "multiplication not working");
    if (fabs(r.bryan.bryan[i] - T3D(0., 0., 0., 4., 4., 4.).bryan.bryan[i]) >1e-10)
      errx(2, "multiplication not working");
  }
  warnx("multiplication ok");


  /* --- inversion --------------------------------------------------- */

  s = t3dMatrixIdentity;
  *t = t3dMatrixIdentity;
  s.invert();
  for(i=0;i<16;i++) if (t->matrix[i] != s.matrix[i])
    errx(2, "inversion not working");
  warnx("inversion ok");

  s = ~s;
  for(i=0;i<16;i++) if (t->matrix[i] != s.matrix[i])
    errx(2, "inversion not working");
  warnx("inversion ok");

  t3dInvert(&s);
  for(i=0;i<16;i++) if (t->matrix[i] != s.matrix[i])
    errx(2, "C-call not working");
  warnx("C-call ok");

  b = s * a;
  for(i=0;i<4;i++) if (b[i] != a[i])
    errx(2, "vector transofrmation not working");
  warnx("vector transformation ok");

  /* --- normalization ----------------------------------------------- */

  a = V3D(2., 2., 2.);
  if (fabs((a.normalize()).norm() - 1.) > 1e-10)
    errx(2, "vector normalization not working");
  warnx("vector normalization ok");

  /* --- dot product ---------------------------------------------- */

  a = V3D(3. , -4., 5.);
  b = V3D(-3., 4., 5.);
  double d = a*b;
  if (fabs(d) > 1e-10)
    errx(2, "dot product not working");
  warnx("dot product ok");

  /* --- vector product ---------------------------------------------- */

  a = V3D(1., 0., 0.);
  b = V3D(0., 1., 0.);
  a = a^b;
  if (fabs((a.normalize()).norm() - 1.) > 1e-10 ||
      fabs(a[2] - 1.) > 1e-10)
    errx(2, "vector product not working");
  warnx("vector product ok");
  return 0;
}
