/*	$LAAS: t3dcopy.c,v 1.2 2005/08/31 21:56:52 sara Exp $ */

/*
 * Copyright (c) 2000-2004 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdio.h>
#include <string.h>

#include "t3d/t3d.h"
#include "t3d/r3d.h"
#include "t3d_internal.h"


/* --- t3dCopyIn ----------------------------------------------------- */

/** \brief Copy T3D.
 *
 * Copy src in cpy. We do not copy more bytes than what the type of the
 * T3D says. The function return a pointer on the copy.
 *
 * \param[out] cpy	Destination.
 * \param[in] src	T3D to copy.
 *
 * \return cpy.
 */

T3D *
t3dCopyIn(T3D *cpy, const T3D *src)
{
  if (!t3dOk(src) || cpy == NULL) { t3dWarn(cpy); return cpy; }

  switch(src->type) {
    case T3D_BRYAN:	memcpy(cpy, src, sizeof(T3DEuler));	break;
    case T3D_VECTOR:	memcpy(cpy, src, sizeof(T3DVector));	break;
    case T3D_MATRIX:	memcpy(cpy, src, sizeof(T3DMatrix));	break;

    case T3D_U_BRYAN:	memcpy(cpy, src, sizeof(T3DUBryan));	break;
    case T3D_U_MATRIX:	memcpy(cpy, src, sizeof(T3DUMatrix));	break;

    default:
      return t3dSetError(cpy, T3D_ENOTIMPLEMENTED);
  }

  return cpy;
}


/* --- r3dCopyIn ----------------------------------------------------- */

/** \brief Copy R3D.
 *
 * Copy src in cpy. We do not copy more bytes than what the type of the
 * R3D says. The function return a pointer on the copy.
 *
 * \param[out] cpy	Destination.
 * \param[in] src	R3D to copy.
 *
 * \return cpy.
 */

R3D *
r3dCopyIn(R3D *cpy, const R3D *src)
{
  return (R3D *)t3dCopyIn((T3D *)cpy, (T3D *)src);
}
