/* 
 * Copyright (c) 2000-2007 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <iostream>

#include "t3d/t3d.h"
#include "t3d/r3d.h"
#include "t3d_internal.h"


/* --- Local data ---------------------------------------------------- */

static int	t3d_ostream(void *cookie, const char *buf, int len);


/* --- operator<< ---------------------------------------------------- */

/** \brief Output a ASCII T3D description to a stream
 *
 * \param[in] s		Stream to dump on.
 * \param[in] t3d	T3D to dump.
 *
 * \return reference on the output stream.
 */

std::ostream &
operator<<(std::ostream &s, const T3D &t3d)
{
  t3dSPrint(&t3d, t3d_ostream, &s);
  return s;
}

/** \brief Output a ASCII R3D description to a stream
 *
 * \param[in] s		Stream to dump on.
 * \param[in] r3d	R3D to dump.
 *
 * \return reference on the output stream.
 */

std::ostream &
operator<<(std::ostream &s, const R3D &r3d)
{
  t3dSPrint((T3D *)&r3d, t3d_ostream, &s);
  return s;
}

/** \brief Output a ASCII V3D description to a stream
 *
 * \param[in] s		Stream to dump on.
 * \param[in] v3d	V3D to dump.
 *
 * \return reference on the output stream.
 */

std::ostream &
operator<<(std::ostream &s, const V3D &v3d)
{
  v3dSPrint(&v3d, t3d_ostream, &s);
  return s;
}


/* --- t3d_ostream --------------------------------------------------- */

static int
t3d_ostream(void *cookie, const char *buf, int len)
{
  static_cast <std::ostream *>(cookie)->write(buf, len);
  return len;
}
