/*	$LAAS: t3d.c,v 1.2 2005/08/31 21:56:52 sara Exp $ */

/*
 * Copyright (c) 2000-2004 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>

#include "t3d/t3d.h"
#include "t3d_internal.h"

#ifdef __cplusplus
namespace detail{
	const double id_4x4[16] = { 1.0, 0.0, 0.0,     0.0,
		0.0, 1.0, 0.0,     0.0,
		0.0, 0.0, 1.0,     0.0,
		0.0, 0.0, 0.0,     1.0 };

	const double zeros6[6] = {
		0.0, 0.0, 0.0,
		0.0, 0.0, 0.0
	};

	const double zeros21[21] = {
		0.0, 0.0, 0.0,	0.0, 0.0, 0.0,
		0.0, 0.0, 0.0,	0.0, 0.0, 0.0,
		0.0, 0.0, 0.0,	0.0, 0.0, 0.0,
		0.0, 0.0, 0.0
	};
}

const T3D t3dMatrixIdentity  (T3D_MATRIX,   0, T3D_EOK,detail::id_4x4);
const T3D t3dUMatrixIdentity (T3D_U_MATRIX,   0, T3D_EOK,detail::id_4x4, detail::zeros21);
const T3D t3dBryanIdentity   (T3D_BRYAN,    0, T3D_EOK,detail::zeros6);
const T3D t3dUBryanIdentity  (T3D_U_BRYAN,    0, T3D_EOK,detail::zeros6, detail::zeros21);
const T3D t3dVectorIdentity  (T3D_VECTOR,   0, T3D_EOK,detail::zeros6);
const T3D t3dUVectorIdentity (T3D_U_VECTOR, 0, T3D_EOK,detail::zeros6, detail::zeros21);
#else
const T3D t3dMatrixIdentity = { .matrix = {
    T3D_MATRIX, 0, T3D_EOK,
    { 1.0, 0.0, 0.0,     0.0,
      0.0, 1.0, 0.0,     0.0,
      0.0, 0.0, 1.0,     0.0,
      0.0, 0.0, 0.0,     1.0 }
  }};

const T3D t3dUMatrixIdentity = { .umatrix = {
    T3D_U_MATRIX, 0, T3D_EOK,
    { 1.0, 0.0, 0.0,     0.0,
      0.0, 1.0, 0.0,     0.0,
      0.0, 0.0, 1.0,     0.0,
      0.0, 0.0, 0.0,     1.0 },

    { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
           0.0, 0.0, 0.0, 0.0, 0.0,
                0.0, 0.0, 0.0, 0.0,
                     0.0, 0.0, 0.0,
                          0.0, 0.0,
                               0.0 }
  }};

const T3D t3dBryanIdentity = { .bryan = {
    T3D_BRYAN, 0, T3D_EOK,
    {{ 0.0, 0.0, 0.0,     0.0, 0.0, 0.0 }}
  }};

const T3D t3dUBryanIdentity = { .ubryan = {
    T3D_U_BRYAN, 0, T3D_EOK,
    {{ 0.0, 0.0, 0.0,     0.0, 0.0, 0.0 }},

    { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
           0.0, 0.0, 0.0, 0.0, 0.0,
                0.0, 0.0, 0.0, 0.0,
                     0.0, 0.0, 0.0,
                          0.0, 0.0,
                               0.0 }
  }};

const T3D t3dVectorIdentity = { .vector = {
    T3D_VECTOR, 0, T3D_EOK,
    {{ 0.0, 0.0, 0.0,     0.0, 0.0, 0.0 }}
  }};

const T3D t3dUVectorIdentity = { .uvector = {
    T3D_U_VECTOR, 0, T3D_EOK,
    {{ 0.0, 0.0, 0.0,     0.0, 0.0, 0.0 }},

    { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
           0.0, 0.0, 0.0, 0.0, 0.0,
                0.0, 0.0, 0.0, 0.0,
                     0.0, 0.0, 0.0,
                          0.0, 0.0,
                               0.0 }
  }};

#endif /*__cplusplus*/

/* --- t3dNew -------------------------------------------------------- */

/** \brief Malloc a new initialized structure for a tr3d.
 *
 * Calls t3dInit() with an malloc'ed structure. Storage must be released
 * with t3dDelete().
 *
 * \param[in] type	Type of internal representation.
 * \param[in] flags	Flags for special behaviors.
 *
 * \return A pointer to the initialized T3D.
 */

T3D *
t3dNew(T3DType type, int flags)
{
   T3D *new_T3D;

   if (flags & T3D_MINIMAL_SIZE) {
     if (flags & T3D_ALLOW_CONVERSION) return NULL;
     switch (type) {
       case T3D_MATRIX:     new_T3D = (T3D*)malloc(sizeof(T3DMatrix));  break;
       case T3D_U_MATRIX:   new_T3D = (T3D*)malloc(sizeof(T3DUMatrix)); break;
       case T3D_BRYAN:      new_T3D = (T3D*)malloc(sizeof(T3DBryan));   break;
       case T3D_U_BRYAN:    new_T3D = (T3D*)malloc(sizeof(T3DUBryan));  break;
       case T3D_VECTOR:     new_T3D = (T3D*)malloc(sizeof(T3DVector));  break;
       case T3D_U_VECTOR:   new_T3D = (T3D*)malloc(sizeof(T3DUVector));  break;
     }
   } else new_T3D = (T3D*)malloc(sizeof(T3D));
   if (new_T3D == NULL) return NULL;

   return t3dInit(new_T3D, type, flags);
}


/* --- t3dInit ------------------------------------------------------- */

/** \brief Initialize a T3D with the type "type".
 *
 * The size of the new structure is compatible with any type of
 * representation except if the T3D_MINIMAL_SIZE flag is present (in this
 * case the allocated size is minimal).
 *
 * \param[in] t3d	Pointer to an allocated T3D.
 * \param[in] type	Type of internal representation.
 * \param[in] flags	Flags for special behaviors.
 *
 * \return A pointer to the initialized T3D (equal to `t3d' parameter).
 */

T3D *
t3dInit(T3D *t3d, T3DType type, int flags)
{
   if ((flags & T3D_MINIMAL_SIZE) && (flags & T3D_ALLOW_CONVERSION))
     return t3dSetError(t3d, T3D_EBADFLAGS);

   t3d->err = T3D_EBADTYPE;
   switch(type) {
     case T3D_MATRIX:	t3dCopyIn(t3d, &t3dMatrixIdentity);	break;
     case T3D_U_MATRIX:	t3dCopyIn(t3d, &t3dUMatrixIdentity);	break;
     case T3D_BRYAN:	t3dCopyIn(t3d, &t3dBryanIdentity);	break;
     case T3D_U_BRYAN:	t3dCopyIn(t3d, &t3dUBryanIdentity);	break;
     case T3D_VECTOR:	t3dCopyIn(t3d, &t3dVectorIdentity);	break;
     case T3D_U_VECTOR:	t3dCopyIn(t3d, &t3dUVectorIdentity);	break;
   }
   if (!t3dOk(t3d)) return NULL;

   t3d->flags = flags;
   return t3d;
}


/*
 * t3dDup ---------------------------------------------------------------
 *
 * Malloc a new structure for a tr3d and initialize it with t3d
 */

T3D *
t3dDup(T3D *t3d)
{
   if (!t3dOk(t3d)) { t3dWarn(t3d); return NULL; }

   return t3dCopyIn(t3dNew(t3d->type, t3d->flags), t3d);
}


/*
 * t3dDelete ------------------------------------------------------------
 *
 * Free an malloc'ed T3D.
 */

void
t3dDelete(T3D *t3d)
{
   if (!t3dOk(t3d)) { t3dWarn(t3d); return; }

   free(t3d);
}
