/* 
 * Copyright (c) 2007 Anthony Mallet                  --  Thu Feb  1 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * $Id: h-bsd2.h,v 1.1.1.1 2005/09/25 22:25:52 tho Exp $
 */
#ifndef H_T3D_INTERNAL
#define H_T3D_INTERNAL

extern const double t3dmid3[9];

#ifdef __cplusplus
extern "C" {
#endif

  void	t3dConvertToMatrix(T3D *t3d);
  void	t3dConvertToEuler(T3D *t3d);
  void	t3dConvertToVector(T3D *t3d);

  void	t3dEulerOfMatrix(double *mat, double *euler);
  void	t3dEulerAnglesOfMatrix(double *mat, double *euler);
  void	t3dVectorOfMatrix(double *mat, double *vector);
  void	t3dMatrixOfEuler(double *euler, double *mat);
  void	t3dMatrixOfVector(double *vector, double *mat);

  void	t3dDerivativeOfEuler(double *euler, double *dRa, double *dRb,
		double *dRg);
  void	t3dDerivativeOfVector(double *vector, double *dRx, double *dRy,
		double *dRz);
  void	t3dRotationOfMatrix(double *m, double *r);

  int	t3d_dm_fill(double *mat, int l, int c, ...);
  int	t3d_dm_blocfill(double *m, int l, int c, int lb, int cb, ...);
  int	t3d_dm_mult(const double *m1, const double *m2, double *m,
		int l1, int c1, int l2, int c2);
  int	t3d_dm_trans_3x3(double *m);
  int	t3d_dms_plus_6x6(double *m1, double *m2, double *m);
  int	t3d_dmfsf_jmult_6x6(double *m1, double *m2, double *m);

  int	t3dSPrint(const T3D *t3d, int (*write)(void *, const char *, int),
		void *cookie);
  int	v3dSPrint(const V3D *v3d, int (*write)(void *, const char *, int),
		void *cookie);

#ifdef __cplusplus
}
#endif

#define t3dMixedType(a, b) (((unsigned)(a) & 0xff)<<8)|((unsigned)(b) & 0xff)



/* --- doxygen global documentation ---------------------------------- */

/** \mainpage
 * This is libt3d, a library for manipulating frames, 3d transformations,
 * 3d rotations and 3d vectors.
 *
 * The library has a C and a C++ API:
 * \li \ref C \li \ref CXX
 */

/**
 * \defgroup C C API
 * \defgroup CXX C++ API
 * \defgroup deprecated Deprecated functions and types
 */

#endif /* H_T3D_INTERNAL */
